/*
 *  Block device elevator/IO-scheduler.
 *
 *  Copyright (C) 2000 Andrea Arcangeli <andrea@suse.de> SuSE
 *
 * 30042000 Jens Axboe <axboe@suse.de> :
 *
 * Split the elevator a bit so that it is possible to choose a different
 * one or even write a new "plug in". There are three pieces:
 * - elevator_fn, inserts a new request in the queue list
 * - elevator_merge_fn, decides whether a new buffer can be merged with
 *   an existing request
 * - elevator_dequeue_fn, called when a request is taken off the active list
 *
 * 20082000 Dave Jones <davej@suse.de> :
 * Removed tests for max-bomb-segments, which was breaking elvtune
 *  when run without -bN
 *
 * Jens:
 * - Rework again to work with bio instead of buffer_heads
 * - loose bi_dev comparisons, partition handling is right now
 * - completely modularize elevator setup and teardown
 *
 */
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/blkdev.h>
#include <linux/elevator.h>
#include <linux/bio.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/compiler.h>
#include <linux/delay.h>
#include <linux/blktrace_api.h>

#include <asm/uaccess.h>

static DEFINE_SPINLOCK(elv_list_lock);
static LIST_HEAD(elv_list);

/*
 * can we safely merge with this request?
 */
inline int elv_rq_merge_ok(struct request *rq, struct bio *bio)
{
}
EXPORT_SYMBOL(elv_rq_merge_ok);

static inline int elv_try_merge(struct request *__rq, struct bio *bio)
{
}

static struct elevator_type *elevator_find(const char *name)
{
}

static void elevator_put(struct elevator_type *e)
{
}

static struct elevator_type *elevator_get(const char *name)
{
}

static void *elevator_init_queue(request_queue_t *q, struct elevator_queue *eq)
{
}

static void elevator_attach(request_queue_t *q, struct elevator_queue *eq,
			   void *data)
{
}

static char chosen_elevator[16];

static int __init elevator_setup(char *str)
{
}

__setup("elevator=", elevator_setup);

static struct kobj_type elv_ktype;

static elevator_t *elevator_alloc(struct elevator_type *e)
{
}

static void elevator_release(struct kobject *kobj)
{
}

int elevator_init(request_queue_t *q, char *name)
{
}

void elevator_exit(elevator_t *e)
{
}

/*
 * Insert rq into dispatch queue of q.  Queue lock must be held on
 * entry.  If sort != 0, rq is sort-inserted; otherwise, rq will be
 * appended to the dispatch queue.  To be used by specific elevators.
 */
void elv_dispatch_sort(request_queue_t *q, struct request *rq)
{
}

int elv_merge(request_queue_t *q, struct request **req, struct bio *bio)
{
}

void elv_merged_request(request_queue_t *q, struct request *rq)
{
}

void elv_merge_requests(request_queue_t *q, struct request *rq,
			     struct request *next)
{
}

void elv_requeue_request(request_queue_t *q, struct request *rq)
{
}

static void elv_drain_elevator(request_queue_t *q)
{
}

void elv_insert(request_queue_t *q, struct request *rq, int where)
{
}

void __elv_add_request(request_queue_t *q, struct request *rq, int where,
		       int plug)
{
}

void elv_add_request(request_queue_t *q, struct request *rq, int where,
		     int plug)
{
}

static inline struct request *__elv_next_request(request_queue_t *q)
{
}

struct request *elv_next_request(request_queue_t *q)
{
}

void elv_dequeue_request(request_queue_t *q, struct request *rq)
{
}

int elv_queue_empty(request_queue_t *q)
{
}

struct request *elv_latter_request(request_queue_t *q, struct request *rq)
{
}

struct request *elv_former_request(request_queue_t *q, struct request *rq)
{
}

int elv_set_request(request_queue_t *q, struct request *rq, struct bio *bio,
		    gfp_t gfp_mask)
{
}

void elv_put_request(request_queue_t *q, struct request *rq)
{
}

int elv_may_queue(request_queue_t *q, int rw, struct bio *bio)
{
}

void elv_completed_request(request_queue_t *q, struct request *rq)
{
}

#define to_elv(atr) container_of((atr), struct elv_fs_entry, attr)

static ssize_t
elv_attr_show(struct kobject *kobj, struct attribute *attr, char *page)
{
}

static ssize_t
elv_attr_store(struct kobject *kobj, struct attribute *attr,
	       const char *page, size_t length)
{
}

static struct sysfs_ops elv_sysfs_ops = {
	.show	= elv_attr_show,
	.store	= elv_attr_store,
};

static struct kobj_type elv_ktype = {
	.sysfs_ops	= &elv_sysfs_ops,
	.release	= elevator_release,
};

int elv_register_queue(struct request_queue *q)
{
}

static void __elv_unregister_queue(elevator_t *e)
{
}

void elv_unregister_queue(struct request_queue *q)
{
}

int elv_register(struct elevator_type *e)
{
}
EXPORT_SYMBOL_GPL(elv_register);

void elv_unregister(struct elevator_type *e)
{
}
EXPORT_SYMBOL_GPL(elv_unregister);

/*
 * switch to new_e io scheduler. be careful not to introduce deadlocks -
 * we don't free the old io scheduler, before we have allocated what we
 * need for the new one. this way we have a chance of going back to the old
 * one, if the new one fails init for some reason.
 */
static int elevator_switch(request_queue_t *q, struct elevator_type *new_e)
{
}

ssize_t elv_iosched_store(request_queue_t *q, const char *name, size_t count)
{
}

ssize_t elv_iosched_show(request_queue_t *q, char *name)
{
}

EXPORT_SYMBOL(elv_dispatch_sort);
EXPORT_SYMBOL(elv_add_request);
EXPORT_SYMBOL(__elv_add_request);
EXPORT_SYMBOL(elv_next_request);
EXPORT_SYMBOL(elv_dequeue_request);
EXPORT_SYMBOL(elv_queue_empty);
EXPORT_SYMBOL(elevator_exit);
EXPORT_SYMBOL(elevator_init);
