#include <linux/kernel.h>
#include <linux/module.h>
#include <rtai.h>
#include <rtai_sched.h>
#include <linux/bootmem_tools.h>

MODULE_DESCRIPTION("LRTAI trivial sample application");
MODULE_AUTHOR("Michael Heimpold <michael.heimpold@s2000.tu-chemnitz.de>");
MODULE_LICENSE("GPLv2");

#define TICK_TIME 1300000000
#define TASKS 1

static RT_TASK mytasks[TASKS];
static int periods[TASKS] = { 10 };

static void mythread(long nr)
{
	while (1) {
		dump_bootmem();
		rt_task_wait_period();
	}
}

static int __init myapp_init(void)
{
	RTIME starttime;
	int period, i;
	
	for (i = 0; i < TASKS; ++i)
		rt_task_init(&mytasks[i],
			     mythread,
			     i,
			     PAGE_SIZE,
			     RT_SCHED_LOWEST_PRIORITY,
			     0,
			     NULL);
	
	period = start_rt_timer(nano2count(TICK_TIME));
	starttime = rt_get_time() + 10 * period;
	
	for (i = 0; i < TASKS; ++i)
		rt_task_make_periodic(&mytasks[i],
				      starttime,
				      periods[i] * period);

	printk("myapp2: %u task(s) created.\n", TASKS);
	return 0;
}

static void __exit myapp_exit(void)
{
	int i;
	stop_rt_timer();
	
	for (i = 0; i < TASKS; ++i)
		rt_task_delete(&mytasks[i]);

	printk("myapp2: %u tasks deleted.\n", TASKS);
}

module_init(myapp_init);
module_exit(myapp_exit);
