#include <linux/kernel.h>
#include <linux/module.h>
#include <rtai.h>
#include <rtai_config.h>
#include <rtai_malloc.h>

MODULE_DESCRIPTION("Sample application which allocates a private heap");
MODULE_AUTHOR("Michael Heimpold <michael.heimpold@s2000.tu-chemnitz.de>");
MODULE_LICENSE("GPLv2");

#define HEAP_SIZE (512 * 1024)

rtheap_t heap;
void *heapaddr = NULL;

static int __init heap_init(void)
{
	if (!(heapaddr = kmalloc(HEAP_SIZE, GFP_KERNEL))) {
		printk("myapp: kmalloc failed.\n");
		return 1;
	}
	printk("myapp: heap inited.\n");
	return 0;
}

#ifdef CONFIG_LRTAI
fs_initcall(heap_init);
#endif

static int __exit heap_exit(void)
{
	/* Either use rtheap_destroy or kfree; never both! */
	if (heapaddr)
		kfree(heapaddr);
	return 0;
}

/*
 Rest of module implementation
 */

static int __init myapp_init(void)
{
	/*
	   ...
	 */
	int rv;
#ifndef CONFIG_LRTAI
	if (!heap_init()) {
		printk("myapp: could not init heap.\n");
		return 1;
	}
#endif
	if (heapaddr
	    && (rv = rtheap_init(&heap, heapaddr, HEAP_SIZE, PAGE_SIZE))) {
		printk("myapp: rtheap_init failed with %d.\n", rv);
		kfree(heapaddr);
		return 1;
	}
	/*
	   Rest of initialization
	 */
	printk("myapp: loaded.\n");
	return 0;
}

static void __exit myapp_exit(void)
{
	/*
	   Rest of destructor
	 */
	if (!heap_exit()) {
		printk("myapp: could not destroy heap.\n");
	}
	printk("myapp: unloaded.\n");
}

module_init(myapp_init);
module_exit(myapp_exit);
