#include <linux/init.h>
#include <linux/mmzone.h>
#include <linux/bootmem.h>
#include <linux/bootmem_tools.h>
#include <rtai_bootmem.h>

/* ghi == global heap init */

int __init rtai_ghi_greedy(rtheap_t * heap, u_long pagesize,
			   u_long extentsize, int *_heapsize)
{
	unsigned long flags;
	bootmem_data_t *bdata = NODE_DATA(0)->bdata;
	unsigned long freeblocks, heapsize, minfree, maxfree, gcd;
	u_long hdrsize, pmapsize, shiftsize, pageshift;
	rtextent_t *extent;
	int n;
	u_long init_size = 0;


	if ((extentsize < RTHEAP_MINEXTSIZE) ||
	    (extentsize > RTHEAP_MAXEXTSIZE))
		return RTHEAP_PARAM;

	spin_lock_irqsave(&bootmem_lock, flags);

#ifdef CONFIG_RTAI_GHI_GREEDY_DEBUG
	printk("Bootmem before rtai_ghi_greedy:\n");
	dump_bootmem_core();
#endif

	bootmem_freeinfo(&freeblocks, &heapsize, &minfree, &maxfree, &gcd);

	printk
	    ("RTAI[malloc]: ghi_greedy: blocks=%lu, sum=%lu(%lu), min=%lu, max=%lu, gcd=%lu\n",
	     freeblocks, heapsize, heapsize << PAGE_SHIFT, minfree,
	     maxfree, gcd);

	/* Attention: heapsize is the count of pages which is free. This does not
	 * mean that we can allocate all the space when using extentsize.
	 * So interprete heapsize as upper bound in the following code!
	 * (Some fields could be larger than necessary because of this.)
	 */

	/* heapsize is needed as byte count, freeinfo returns pages */
	heapsize <<= PAGE_SHIFT;

	if ((pagesize < (1 << RTHEAP_MINLOG2)) ||
	    (pagesize > (1 << RTHEAP_MAXLOG2)) ||
	    (pagesize & (pagesize - 1)) != 0 ||
	    /* the following line should be caught by RTHEAP_MINEXTSIZE */
	    /* heapsize <= sizeof(rtextent_t) || */
	    heapsize > RTHEAP_MAXEXTSZ || (heapsize & (pagesize - 1)) != 0) {
		spin_unlock_irqrestore(&bootmem_lock, flags);
		return RTHEAP_PARAM;
	}

	pmapsize =
	    ((heapsize -
	      sizeof(rtextent_t)) * sizeof(u_char)) / (pagesize +
						       sizeof(u_char));

	hdrsize =
	    (sizeof(rtextent_t) + pmapsize + RTHEAP_MINALIGNSZ -
	     1) & ~(RTHEAP_MINALIGNSZ - 1);

	if (hdrsize + 2 * pagesize > heapsize) {
		spin_unlock_irqrestore(&bootmem_lock, flags);
		return RTHEAP_PARAM;
	}

	/* Compute the page shiftmask from the page size (i.e. log2 value). */
	for (pageshift = 0, shiftsize = pagesize; shiftsize > 1;
	     shiftsize >>= 1, pageshift++);

	heap->flags = 0;
	heap->pagesize = pagesize;
	heap->pageshift = pageshift;
	heap->hdrsize = hdrsize;
	heap->extentsize = extentsize;
	heap->npages = (heap->extentsize - hdrsize) >> pageshift;
	heap->maxcont = heap->npages * pagesize;
	heap->ubytes = 0;
	INIT_LIST_HEAD(&heap->extents);
	spin_lock_init(&heap->lock);

	for (n = 0; n < RTHEAP_NBUCKETS; n++) {
		heap->buckets[n] = NULL;
	}

	while ((extent = (rtextent_t *)
		____alloc_bootmem_core(bdata, heap->extentsize, 0, 0, 0))) {

		heapsize -= heap->extentsize;
		if (heapsize < RTHEAP_RESERVE) {
			__free_bootmem_core(bdata, virt_to_phys(extent),
					    heap->extentsize);
			break;
		}

		init_extent(heap, extent);
		list_add_tail(&extent->link, &heap->extents);
		init_size += heap->extentsize;
	}

#ifdef CONFIG_RTAI_GHI_GREEDY_DEBUG
	printk("Bootmem after rtai_ghi_greedy:\n");
	dump_bootmem_core();
#endif

	spin_unlock_irqrestore(&bootmem_lock, flags);

	if (_heapsize)
		*_heapsize = init_size;

	return 0;
}
