#!/bin/bash
# This dump script checks various values to determine
# if a lowered CONFIG_PHYSICAL_START will crash with
# the compressed part in memory.
#
# Call this script from build directory
#
if [ ! -f .config ] ; then
    echo "Cwd seems not to be a build directory. Exiting..."
    exit 1
fi
KERNELOUTPUT=../build

# set $CONFIG_PHYSICAL_START
DOTCONFIG=$KERNELOUTPUT/.config
. $DOTCONFIG

# virtual pg0 address
PG0=$(grep pg0 $KERNELOUTPUT/System.map | awk '{print "0x" $1}')

echo "PHYSICAL_START:       $CONFIG_PHYSICAL_START"
echo "PAGE_OFFSET:          $CONFIG_PAGE_OFFSET"
echo "PG0:                  $PG0"

SECVAL=$[16 * 1024]
echo "SECVAL:               $SECVAL"

IMSIZE=$[$PG0 - $CONFIG_PAGE_OFFSET - $CONFIG_PHYSICAL_START + $SECVAL]

echo "SIZE:                 $IMSIZE"

STOPADDR="0x90200"
STARTADDR=$[(($STOPADDR - $IMSIZE) / 4096) * 4096]

echo "STARTADDR (ALIGNED):  0x$(printf %x $STARTADDR) ($STARTADDR)"

SIZEVMLINUX=$(stat -c%s "$KERNELOUTPUT/arch/i386/boot/vmlinux.bin")

echo "COMPR. PART:          $SIZEVMLINUX"

COMPRESSEDENDS=$[(($SIZEVMLINUX + 0x1000) / 4096) * 4096 + 4096 -1]
echo "COMPR. END (ALIGNED): 0x$(printf %x $COMPRESSEDENDS) ($COMPRESSEDENDS)"

SECVAL2=$[8 * 1024]
echo "SECVAL2:              $SECVAL"

SAFESTART=$[COMPRESSEDENDS + 1 + SECVAL2]

echo "REMAINING SPACE:      $[$STARTADDR - $SAFESTART] bytes"

if [ "$COMPRESSEDENDS" -ge "$STARTADDR" ] ; then
	echo "ERROR. AREAS OVERLAP! CONFIGURATION WILL NOT WORK!"
else
	echo "OK. CONFIGURATION SHOULD WORK"
	echo "SAFE RANGE: 0x$(printf %x $SAFESTART) - 0x$(printf %x $STARTADDR)"
fi
